import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;
import dmoisan.xml.tvschedule.*;
/**
* ScXML -- Creates Scala Lingua(TM) scripts from TVSCHEDULE formattted
*             XML files that contain TV schedules for one or more
*             channels for the week (or weeks.)  
*
* <p>Main class:  Takes the file spec of the input TVSCHEDULE file
* and the output Lingua script, instantiates the parser and
* passes the file names on to TVScheduleHandler</p>
*
* <br/>Usage:
*
* <br/><code>java ScXML &lt;inputXMLfile&gt; &lt;templatefile&gt;
* &lt;outputscript&gt; -singleoutput</code>
*
* <p>where <code>inputXML</code> is the TVSCHEDULE XML file, 
* <code>templatefile</code> contains the Scala template script and 
* <code>outputscript</code> is the Scala Lingua script file to
* be generated.  If the <code>-singleoutput</code> flag is used, 
* all output goes to <code>outputscript</code>  Otherwise, all
* output goes to several files of the form 
* <code>outputscript+channel number</code>, depending on how many
* program channels are listed in the original file.</p>
* 
* <p>Copyrights:  Scala, Scala Multimedia, and Lingua are TM and (C) of Scala, Inc.
* NO CONNECTION TO SCALA, INC. IS IMPLIED.
* </p>
* <p>Permission is granted to use this code for commercial or non-commercial
* purposes.  NO WARRANTY IS IMPLIED.</p>
* @author David Moisan
* @version 0.9 26 May 1999
* @see dmoisan.xml.tvschedule.ScScript
* @see dmoisan.xml.tvschedule.TVScheduleHandler
* @see dmoisan.text.TemplateStream
*/
public class ScXML {
private static boolean singleoutput=false;
private static final boolean DEBUG=false;

     public static void main (String args[]) throws Exception {
          

          // Argument check    
     
          
          if (args.length<3) {
               UsageCheck();
               System.exit(1);
               }
          else if ((args.length==4) && 
               (args[3].equals("-singleoutput"))) {
               singleoutput=true;
               }

          TVScheduleOutputObject ScS;
          TVScheduleHandler TVH; 
          //
          // Start parsing
          //
          try {
               ScS = new ScScript(args[1], args[2], singleoutput);
               TVH = new TVScheduleHandler(args[0],ScS);                        
               TVH.init();
               }
          catch (SAXException e) {
               System.out.println("Parsing Error: ");
               System.out.println(e.getException().getMessage());
               if (DEBUG)
                    e.getException().printStackTrace();
               }
          catch (IOException e) {
               System.out.println("File error: "+e.getMessage());    
               }     
          }
     
     private static void UsageCheck() {
          System.out.println("ScXML Usage:");
          System.out.println("java ScXML <tvschedulexmlfile> <templatefile> <outputscript> [-singleoutput]");
          System.out.println("");
          System.out.println("-singleoutput       Generates multiple");
          System.out.println("                    schedules to a single");
          System.out.println("                    output file");                                    
          return;
     }
}
