package dmoisan.xml.tvschedule;
import java.io.*;
import java.util.StringTokenizer;
import dmoisan.text.*;

/**
*
* ScScript  -- Constructor and methods for writing
*              Scala Lingua (TM) code from TVSCHEDULE XML elements
* 
* <p>Copyrights:  Scala, Scala Multimedia, and Lingua are TM and (C) of Scala, Inc.
* NO CONNECTION TO SCALA, INC. IS IMPLIED.
* </p>
* <p>Permission is granted to use this code for commercial or non-commercial
* purposes.  NO WARRANTY IS IMPLIED.</p>
*
* <p>See <a href="http://www1.shore.net/~dmoisan/comp/xml/scxml/scxmltechnotes.html">
* http://www1.shore.net/~dmoisan/comp/xml/scxml/scxmltechnotes.html</a> for usage information.
* </p>
*
* @version 0.9 26-May-1999
* @author David Moisan
* @see TVScheduleOutputObject
* @see TVScheduleHandler
* @see ProgramItem
* @see dmoisan.text.TemplateStream
*/


public class ScScript extends TVScheduleOutputObject  {
/*
* Class Variables
*/

private static final boolean DEBUG = false;

// 
// Private instance variables
//
//
//delete this line if you're using the Windows version of Scala
private static final String LINESEP = "\n";
//uncomment this line if you're using the Windows version of Scala
//private static final String LINESEP = System.getProperties("line.separator);

private static final String SCFILEHEADER = "V3.0"+LINESEP+LINESEP;
private static final String SCEVENTHEADER = "EVENT \"";
private static final String SCEVENTFOOTER = "END"+LINESEP+LINESEP;
private static final String SCKEYWORDSTART = "<";
private static final String SCKEYWORDEND = ">";

/*
* Instance variables
*/

private String ScriptName;
private TemplateStream Template;
private String TemplateContents;
private String CurrentChannel;
private String CurrentDay;
private boolean singleoutput;
private boolean NewPage;
private int PageNum;
private FileWriter ScriptOutput;

/**
* Public Constructor
*
* @param templatefilename String  Template file
* @param outputscriptname String If singleoutput is true, then
* this is the output file name, otherwise outputscriptname+channelnumber
* (SATV3, for example) is the output file name.
*
* @param singleoutput boolean Single-file-output flag, as described above.
* @exception java.io.IOException Thrown on file I/O errors
*/

     public ScScript(String templatefilename, String outputscriptname,
          boolean singleoutput) throws IOException {
          this.ScriptName=outputscriptname;

          // Read in template file to buffer

          FileReader tempfile = new FileReader(templatefilename);
          BufferedReader templatestream = new BufferedReader(tempfile);

          StringBuffer templatebuf = new StringBuffer("");
          while (templatestream.ready()) {
               templatebuf.append(templatestream.readLine());
               templatebuf.append(LINESEP);
               }
          
          TemplateContents = templatebuf.toString();
          this.Template = new TemplateStream(TemplateContents);
          this.singleoutput = singleoutput;
          // Initialize TemplateStream

          templatestream.close();
          this.Template.setTemplateDelim
               (SCEVENTHEADER, SCEVENTFOOTER, 
               SCKEYWORDSTART, SCKEYWORDEND);
          return;
     }
/**
* 
* newSchedule(String schedulename)
*   Intialize new Scala schedule object 
*
* @param schedulename (String) Name of TV schedule NOT USED IN THIS VERSION
*
* @exception TVScheduleParseException  Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O errors
*
*/

public void newSchedule(String schedulename) throws TVScheduleParseException, IOException {
// If there is a single output file, create it here and start
// writing to it

     if (singleoutput) {
          this.newScriptOutput(this.ScriptName);
          }
     return;
     }

/**
* newChannel -- Initialize new channel output object.
*
* @param channel Channel name
*
* @exception TVScheduleParseException  Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O errors
*
*/

public void newChannel(String channelname) throws TVScheduleParseException, IOException {
// If output files are separate, open one here and write to it

     this.CurrentChannel = channelname;
     if (!singleoutput) {
          this.newScriptOutput(this.ScriptName+channelname);
          }
     return;
     }

/**
* setBanner -- Set banner
*
*
* @param banner String Banner text
*
* @exception TVScheduleParseException  Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O errors
*
*/

public void setBanner(String banner) throws TVScheduleParseException, IOException {
     this.setnewTemplate("BANNER"+CurrentChannel);
     boolean replaced=Template.replaceString
               ("BANNER",ScDelim(banner),this.ScriptOutput);
     if (DEBUG) System.out.println("replaced= "+replaced);
     if (!replaced) {
          throw new TVScheduleParseException("BANNER not replaced");
          }
     this.Template.closeTemplateOutput(this.ScriptOutput);
     return;
     
     }

/**
* startDay -- Start programming day. 
*
*
* @param day String Current programming day
*
* @exception TVScheduleParseException  Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O errors
*
*/
	
public void startDay(String day) throws TVScheduleParseException, IOException {
     this.CurrentDay=day;
     this.NewPage=true;
     return;
     }
/**
* setHoliday 
*   Get the Holiday text from the parser
*
*
* @param holiday String Holiday notification text
*
* @exception TVScheduleParseException  Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O errors
*/

public void setHoliday(String holiday) throws TVScheduleParseException, IOException {
     boolean replacenum, datereplaced, holidayreplaced;
     
     this.setnewTemplate("HOLIDAY"+CurrentChannel);
     datereplaced=this.Template.replaceString
          ("DATE", ScDelim(CurrentDay), ScriptOutput);
     replacenum=this.Template.replaceString
          ("PAGENUM", String.valueOf(this.PageNum), this.ScriptOutput);
     this.PageNum++;
     
     datereplaced=this.Template.replaceString
          ("DATE", ScDelim(CurrentDay), ScriptOutput);
     holidayreplaced=Template.replaceString
          ("HOLIDAY", ScDelim(holiday), ScriptOutput);
     if (!(datereplaced | holidayreplaced | replacenum)) {
          throw new TVScheduleParseException
               ("DATE/HOLIDAY page not replaced!");
               }
     return;     
     }


/**
* newProgram -- Insert new program item
*
*
* @param program ProgramItem Program information
*
* @exception TVScheduleParseException  Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O errors
*
*/

public void newProgram(ProgramItem program) throws TVScheduleParseException, IOException {
     boolean programinserted=false;
     // Start a new page if needed

     if (this.NewPage) {
          this.loadScheduleTemplate();
          }
     // Insert the program item

     programinserted=loadProgram(program);
     
     // If program item not inserted 
     if (!programinserted) {
          this.endDay();
          this.startDay(this.CurrentDay);
          this.loadScheduleTemplate();
          programinserted = this.loadProgram(program);
          this.NewPage=false;
          if (!programinserted) {
               throw new TVScheduleParseException("Unable to load new program data");
               }
          }           
     else {
          this.NewPage=false;
          }
          
     return;
     }
 
/**
*
* endDay -- End programming day.  Usually used to end the current page.  
*
* @exception TVScheduleParseException  Thrown on parsing/semantic
* errors
* @exception IOException Thrown on I/O errors
*
*/

public void endDay() throws TVScheduleParseException, IOException {
     this.Template.closeTemplateOutput(this.ScriptOutput);
     return;
     }

/**
* endChannel -- End Channel.
*     Used to close the current output file.
*
* @exception TVScheduleParseException  Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O errors
*
*/

public void endChannel() throws TVScheduleParseException, IOException {
     if (!singleoutput) {
          ScriptOutput.close();
          }
     return;
     }

/**
* endSchedule -- Close all files
*
* @exception TVScheduleParseException Thrown on parsing/semantic errors
* @exception IOException Thrown on I/O Errors
*
*/

public void endSchedule() throws TVScheduleParseException, IOException {
     if (!singleoutput) {
          ScriptOutput.close();
          }
     return;
     }

/*
* Private instance methods
*
*/

private void newScriptOutput(String name) throws IOException {
     ScriptOutput = new FileWriter(name);
     ScriptOutput.write(SCFILEHEADER); 
     this.PageNum = 1;
     }

/*
* setnewTemplate(String name)
*/

private void setnewTemplate(String name) throws TVScheduleParseException {
     boolean loaded=this.Template.newTemplate(name);
     if (!loaded) {
          throw new 
               TVScheduleParseException(name+" template not loaded");
         }
     }


/** 
* ScDelim -- Strip out stray line feed, tabs and other
* nonprinting characters and escape them with Scala escape 
* codes according to the following list:
*
* <PRE>
*
* /t/     Tab
* /"/     Quote
* \       Newline (equivalent to java escape string "\n")
* </PRE>
*
* @param String chdata String to convert
* @returns String Converted string
*
*/

private static String ScDelim(String chdata) {
     

     //
     // Java delimiting characters
     //

     final String NL = "\n";
     final String CR = "\r";
     final String QUO = "\"";
     final String TAB = "\t";

     //
     // Scala delimiting characters
     //
     
     final String ScNL = "\\";
     final String ScTAB = "/t/";
     final String ScQUO = "/\"/";

     StringTokenizer chdataTokenizer = new StringTokenizer(chdata, NL+CR+QUO+TAB,true);
     StringBuffer chdatabuff = new StringBuffer("");
     String chtoken = "";

     while (chdataTokenizer.hasMoreTokens()) {
          chtoken = chdataTokenizer.nextToken();
          if (chtoken.equals(NL)) {
               chdatabuff.append(ScNL);
               }
          else if (chtoken.equals(TAB)) {
               chdatabuff.append(ScTAB);
               }
          else if (chtoken.equals(QUO)) {
               chdatabuff.append(ScQUO);
               }
          else if (chtoken.equals(CR)) {
               //skip carriage returns
               //and don't emit them to output
               }
          else {
               chdatabuff.append(chtoken);
               // if the token is not a delimiter,
               // then it's text so put it in the buffer
               }
          }
          return chdatabuff.toString();  


     }
    
/**
* loadScheduleTemplate()
*     Load new schedule page template when requested
* @exception IOException
* @exception TVScheduleParseException
*/

private void loadScheduleTemplate() throws IOException, TVScheduleParseException {
     this.setnewTemplate("SCHEDULE"+this.CurrentChannel);
     boolean replacenum, replacedispdate, replacepagedate;
     // Insert the date into the name of the page itself...     
     replacepagedate=this.Template.replaceString
          ("DATE", ScDelim(this.CurrentDay), this.ScriptOutput);

     // Insert a page number (no duplicate page names allowed in Scala!)
     // 
     replacenum=this.Template.replaceString
          ("PAGENUM", String.valueOf(this.PageNum), this.ScriptOutput);
     this.PageNum++;

     //...and insert it again into the page as a display date
     replacedispdate=this.Template.replaceString
          ("DATE", ScDelim(this.CurrentDay), this.ScriptOutput);

     if (!(replacedispdate & replacepagedate)) {
          throw new TVScheduleParseException
               ("DATE not replaced in SCHEDULE template.");
               }
     return;
     }

/**
* loadProgram load program information into output template
*
* @param program ProgramItem  Program information to load into template
* @returns boolean  True if template successfully loaded
*/

private boolean loadProgram(ProgramItem program) throws IOException {

     boolean programreplaced, descriptionreplaced, timereplaced;
     // Try to replace schedule template keywords with
     // program time, title and description.
     //
     
     // load Schedule template if not already loaded

     timereplaced=this.Template.replaceString
          ("TIME",ScDelim(program.Showtime),this.ScriptOutput);
     programreplaced=this.Template.replaceString
          ("TITLE", ScDelim(program.Title),this.ScriptOutput);
     descriptionreplaced=this.Template.replaceString
          ("DESCRIPTION", ScDelim(program.Description), this.ScriptOutput);
     if (DEBUG) System.out.println("timereplaced= "+timereplaced);

     return (timereplaced & programreplaced & descriptionreplaced);
     }   
}
